program I2C_1Wire;


Device = mega128, VCC = 5;

Define_Fuses
  LockBits0      = [];
  FuseBits0      = [BODEN];
  FuseBits1      = [CKOPT, SPIEN, JTAGEN, OCDEN];
  FuseBits2      = [];
  ProgMode       = JTAG;

Import SysTick, SerPort, LCDmultiPort, TWIMaster, BeepPort;

From System Import LongWord, LongInt, Float, Processes, Tasks, Pipes;

Define
  ProcClock      = 16000000;        {Hertz}
  SysTick        = 10;              {msec}
  StackSize      = $0064, iData;
  FrameSize      = $0064, iData;
  Scheduler      = iData;
  TaskStack      = $0020, iData;
  TaskFrame      = $0020;
  
  SerPort        = 19200;
  RxBuffer       = 16, iData;
  TxBuffer       = 16, iData;

  TWIpresc       = TWI_BR400;
  LCDmultiPort   = I2C_TWI;
  LCDrows_M      = 4;               {rows}
  LCDcolumns_M   = 20;              {columns per line}
  LCDtype_M      = 44780;

  BeepPort       = PortG, 4;



Implementation

{$IFNDEF TWIMaster or I2CPort}
{$ShowError 'No TWI or I2C Port imported for OW-Chip'}
{$ENDIF}


{$IDATA}
{--------------------------------------------------------------}
{ Type Declarations }
type
  t_OW_Channel   = (Channel_7 = $87, Channel_6 = $96, Channel_5 = $A5,
  Channel_4      = $B4, Channel_3 = $C3, Channel_2 = $D2,
  Channel_1      = $E1, Channel_0 = $F0);

  t_FamilyCode   =   (DS1990A = $01, DS1991  = $02, DS2404  = $04, DS2405  = $05,
  DS1993         = $06, DS1992  = $08, DS1982  = $09, DS1995  = $0A,
  DS1985         = $0B, DS1996  = $0C, DS1986  = $0F, DS1XS20 = $10,
  DS2406         = $12, DS1983  = $13, DS2430A = $14, DS1954  = $16,
  DS19631        = $1A, DS2423  = $1D, DS2409  = $1F,
  DS2450         = $20, DS1921  = $21, DS1822  = $22, DS2433  = $23,
  DS2415         = $24, DS2438  = $26, DS2417  = $27, DS18B20 = $28,
  DS2890         = $2C, DS2760  = $30, DS2432  = $33, DS2422  = $41);


  t_DS1821_cmd    = (WriteStatusReg = $0C, StopConv = $22, ReadTemperature = $AA, StartConv = $EE);

  t_DS2482_cmd    =  (OWRB = $96, OWWB = $A5, OWRS = $B4, CHSL = $C3, SRP = $E1, DRST = $F0);

  t_Ds2482_Reg    =  (Conf_Reg = $C3, ChanSel_Reg = $D2, ReadData_Reg = $E1, Stat_Reg = $F0);


  t_OW_Global_cmd = (ReadROM = $33, OD_SkipROM = $3C, MatchROM = $55,
                     OD_MatchROM = $69, SkipROM = $CC, SearchROM = $F0);

  t_Mem_cmd       = (WriteScratchpad = $0F,CopyPage=$55, ReadScratchpad = $AA);

  t_Status        = (Wire_Busy, PPD, SD, LL, RST, SBR, TBP, DIR);

  tBitset         = BitSet of t_Status;

  t_Authorization = Record
                     TA1 :Byte;
                     TA2 :Byte;
                     ES: Byte;
                     end;

  t_OW_Serial    =  Record
                      CRC            : Byte;
                      Serial         : Array[0..5] of Byte;
                      FamilyCode     : t_familyCode;
                    end;

  t_Postage      = Record
                     Company_Name   : String[26];
                     Street         : String[22];
                     Country        : String[2];
                     ZipCode        : String[5];
                     City           : String[22];
                   end;

  t_Telecom      = Record
                     Tel_Country_Code: String[2];
                     Tel_Area_Code  : String[5];
                     Tel_Number     : String[8];
                     Tel_Ext        : String[4];
                     Fax_Country_Code: String[2];
                     Fax_Area_Code  : String[5];
                     Fax_Number     : String[8];
                     Fax_Ext        : String[4];
                   end;




  t_Factory      = Record
                     Addresse       : t_Postage;
                     Telefon        : t_Telecom;
                   end;

  t_Customer     = Record
                     Addresse       : t_Postage;
                     Telefon        : t_Telecom;
                   end;

  t_Date         = Record
                     Tag            : String[2];
                     Monat          : String[2];
                     Jahr           : string[4];
                   end;

  t_AddData      = Record
                     HW_Name        : String[11];
                     HW_Haupt       : String[2];
                     HW_Neben       : String[2];
                     HW_Revison     : String[2];
                     HW_Produktion  : t_Date;
                     SW_Name        : String[11];
                     SW_Haupt       : String[2];
                     SW_Neben       : String[2];
                     SW_Revison     : String[2];
                     SW_Produktion  : t_Date;
                   end;

  t_TypeofSensor = (Volumen_Klein = $01, Volumen_Gross = $02, Volumen_Analog = $03,
  Volumen_H2O    = $04);


  t_PnP_Data     = Record
                     TypeofSensor   : t_TypeofSensor;
                   end;

{--------------------------------------------------------------}
{ Const Declarations }
Const
  CRCTable       : Table[0..255] of Byte = (
  0, 94, 188, 226, 97, 63, 221, 131, 194, 156, 126, 32, 163, 253, 31, 65,
  157, 195, 33, 127, 252, 162, 64, 30, 95, 1, 227, 189, 62, 96, 130, 220,
  35, 125, 159, 193, 66, 28, 254, 160, 225, 191, 93, 3, 128, 222, 60, 98,
  190, 224, 2, 92, 223, 129, 99, 61, 124, 34, 192, 158, 29, 67, 161, 255,
  70, 24, 250, 164, 39, 121, 155, 197, 132, 218, 56, 102, 229, 187, 89, 7,
  219, 133, 103, 57, 186, 228, 6, 88, 25, 71, 165, 251, 120, 38, 196, 154,
  101, 59, 217, 135, 4, 90, 184, 230, 167, 249, 27, 69, 198, 152, 122, 36,
  248, 166, 68, 26, 153, 199, 37, 123, 58, 100, 134, 216, 91, 5, 231, 185,
  140, 210, 48, 110, 237, 179, 81, 15, 78, 16, 242, 172, 47, 113, 147, 205,
  17, 79, 173, 243, 112, 46, 204, 146, 211, 141, 111, 49, 178, 236, 14, 80,
  175, 241, 19, 77, 206, 144, 114, 44, 109, 51, 209, 143, 12, 82, 176, 238,
  50, 108, 142, 208, 83, 13, 239, 177, 240, 174, 76, 18, 145, 207, 45, 115,
  202, 148, 118, 40, 171, 245, 23, 73, 8, 86, 180, 234, 105, 55, 213, 139,
  87, 9, 235, 181, 54, 104, 138, 212, 149, 203, 41, 119, 244, 170, 72, 22,
  233, 183, 85, 11, 136, 214, 52, 106, 43, 117, 151, 201, 74, 20, 246, 168,
  116, 42, 200, 150, 21, 75, 169, 247, 182, 232, 10, 84, 215, 137, 107, 53);

  OW_I2C  : boolean  = False;

{--------------------------------------------------------------}
{ Var Declarations }
{$IDATA}
Var
  DS2482_Status  : tBitset;
  I2CAdresse     : Byte;

  a, b, c        : Byte;
  MyCompany     : t_factory;
  TestCompany      : t_factory;
  Th             : Integer;
  Test           : t_OW_Serial;
  Check          : Byte;
  Serial_OK      : Boolean;
  Check_OnLine   : Boolean;

{--------------------------------------------------------------}
{ functions }
Procedure DS2482_SetI2CAdresse(Adresse : Byte);
Begin
  I2CAdresse := Adresse;
end;

function DS2482_GetI2CAdresse(Adresse : Byte) : Byte;
Begin
  Return(I2CAdresse);
end;


Procedure DS2482_Poll_BitLow(Status_Bit : t_Status);
Begin
  repeat                                                       //Polling for busy
    {$IF OW_I2C = True}
    I2CInp(I2CAdresse, DS2482_Status);
    {$ELSE}
    TWIInp(I2CAdresse, DS2482_Status);
    {$ENDIF}
  until not Status_Bit in DS2482_Status;
End;

Procedure DS2482_Poll_BitHi(Status_Bit : t_Status);
Begin
  repeat                                                       //Polling for busy
    {$IF OW_I2C = True}
    I2CInp(I2CAdresse, DS2482_Status);
    {$ELSE}
    TWIInp(I2CAdresse, DS2482_Status);
    {$ENDIF}
  until Status_Bit in DS2482_Status;
End;

{-------------------------Simple DS2482 Code-------------------------}

function DS2482_SelectChannel(Channel : t_OW_Channel) : boolean;
begin
{$IF OW_I2C = True}
  Return(I2COut(I2CAdresse, CHSL, Channel));
{$ELSE}
  Return (TWIOut(I2CAdresse, CHSL, Channel));
{$ENDIF}
end;


function DS2482_Reset : boolean;
begin
  {$IF OW_I2C = True}
  If I2COut(I2CAdresse, DRST) = False then
    Return(False);
  else
    DS2482_Poll_BitHi(Rst);
    Return(True);
  End_If;
{$ELSE}
  If TWIOut(I2CAdresse, DRST) = False then
    Return(False);
  else
    DS2482_Poll_BitHi(Rst);
    Return(True);
  End_If;
{$ENDIF}
end;

function DS2482_WriteByte(Value : Byte) : Boolean;
begin
{$IF OW_I2C = True}
  If I2COut(I2CAdresse, OWWB, Value) = False then
    Return(False);                                                //1Wire is busy
  else
    DS2482_Poll_BitLow(Wire_Busy);
    Return(True);
  end_if;
{$ELSE}
  If TWIOut(I2CAdresse, OWWB, Value) = False then
    Return(False);                                                //1Wire is busy
  else
    DS2482_Poll_BitLow(Wire_Busy);
    Return(True);
  end_if;
{$ENDIF}
end;

function DS2482_ReadRegister(Register : t_Ds2482_Reg ) : Byte;
Var
  Status         : Byte;
Begin
{$IF OW_I2C = True}
  I2COut(I2CAdresse, SRP, Register);
  I2CInp(I2CAdresse, Status);
  Return(Status);
{$ELSE}
  TWIOut(I2CAdresse, SRP, Register);
  TWIInp(I2CAdresse, Status);
  Return(Status);
{$ENDIF}
end;

function DS2482_ReadByte : Byte;
Var
  Dummy          : Byte;
begin
  {$IF OW_I2C = True}
  I2COut(I2CAdresse, OWRB);
  DS2482_Poll_BitLow(Wire_Busy);
  Return(DS2482_ReadRegister(ReadData_Reg));
  {$ELSE}
  TWIOut(I2CAdresse, OWRB);
  DS2482_Poll_BitLow(Wire_Busy);
  Return(DS2482_ReadRegister(ReadData_Reg));
  {$ENDIF}
end;

{--------------------------1-Wire Code---------------------------------}


function OW_Reset : Boolean;
begin
{$IF OW_I2C = True}
  If I2COut(I2CAdresse, OWRS) = False then
    Return(False);                                            //1Wire is busy
  else
    DS2482_Poll_BitLow(Wire_Busy);
    Return(PPD in DS2482_Status);
  end_if;
{$ELSE}
  If TWIOut(I2CAdresse, OWRS) = False then
    Return(False);                                            //1Wire is busy
  else
    DS2482_Poll_BitLow(Wire_Busy);
    Return(PPD in DS2482_Status);
  end_If;
{$ENDIF}
end;


function OW_Write(Value : Byte) : Boolean;
Begin
  Return((OW_Reset = True) and (DS2482_WriteByte(Value) = True));
end;

function OW_Read : Byte;
Begin
  Return(DS2482_ReadByte);
end;

function OW_ReadSingleSerial(Var Serial : t_OW_Serial) : Boolean;
Begin
  If OW_Write(Byte(ReadROM)) = False then
    Return(False);
  Else
    Serial.FamilyCode := t_FamilyCode(OW_Read);
    Serial.Serial[0] := OW_Read;
    Serial.Serial[1] := OW_Read;
    Serial.Serial[2] := OW_Read;
    Serial.Serial[3] := OW_Read;
    Serial.Serial[4] := OW_Read;
    Serial.Serial[5] := OW_Read;
    Serial.CRC := OW_Read;
    Return(True);
  end_If;
end;

function OW_SelectChip(Serial : t_OW_Serial) : Boolean;
Begin
  Return (OW_Write(Byte(MatchROM)) and DS2482_WriteByte(Byte(Serial.FamilyCode))
  and DS2482_WriteByte(Serial.Serial[0]) and DS2482_WriteByte(Serial.Serial[1])
  and DS2482_WriteByte(Serial.Serial[2])and DS2482_WriteByte(Serial.Serial[3])
  and DS2482_WriteByte(Serial.Serial[4]) and DS2482_WriteByte(Serial.Serial[5])
  and DS2482_WriteByte(Serial.crc));
end;

Procedure OW_WritePage(Offset:Word; DataPointer:Pointer);
Var
  Dummy: Byte;
Begin
  DS2482_WriteByte(Byte(WriteScratchpad));
  DS2482_WriteByte(Lo(Offset));
  DS2482_WriteByte(Hi(Offset));
  For dummy := 0 to 31 do
    DS2482_WriteByte(datapointer^);
    Inc(Datapointer);
  End_For;
end;

Procedure OW_ReadPage(Offset:Word; DataPointer:Pointer; VAR Authorization: t_Authorization);
Var
  Dummy: Byte;
Begin
  DS2482_WriteByte(Byte(ReadScratchpad));
  Authorization.TA1:= OW_Read;
  Authorization.TA2:= OW_Read;
  Authorization.ES:= OW_Read;
  If Datapointer > NIL then
    For dummy := 0 to 31 do
      datapointer^ := OW_Read;
      Inc(Datapointer);
    End_For;
  end_If;
end;

{-------------------------Serial CRC Code ----------------------------------}


function OW_CalcCRC(Serial : t_OW_Serial) : Byte;
Var
  CRC            : Byte;
Begin
  CRC:= 0;
  CRC := GetTable(CRCTable, CRC XOR Byte(Serial.FamilyCode));
  CRC := GetTable(CRCTable, CRC XOR Serial.Serial[0]);
  CRC := GetTable(CRCTable, CRC XOR Serial.Serial[1]);
  CRC := GetTable(CRCTable, CRC XOR Serial.Serial[2]);
  CRC := GetTable(CRCTable, CRC XOR Serial.Serial[3]);
  CRC := GetTable(CRCTable, CRC XOR Serial.Serial[4]);
  CRC := GetTable(CRCTable, CRC XOR Serial.Serial[5]);
  Return (CRC);
end;

function OW_MatchSerialCRC(Serial : t_OW_Serial) : Boolean;
Begin
  Return(OW_CalcCRC(Serial) = Serial.CRC);
end;

{-----------------------DS1821 Code-------------------------------------}

function DS1821_Setup : Boolean;
Begin
  Return((OW_Write(Byte(WriteStatusReg)) = True) and (DS2482_WriteByte(0) = True));
end;

function DS1821_StartConv : Boolean;
Begin
  Return(OW_Write(Byte(StartConv)));
end;

function DS1821_StopConv : Boolean;
Begin
  Return(OW_Write(Byte(StopConv)));
end;

function DS1821_GetTemperature : Integer;
Var
  Dummy            : Integer;
  LoByte[@Dummy]   : Byte;
  HiByte[@Dummy+1] : Byte;
Begin
  OW_Write(Byte(ReadTemperature));
  LoByte := OW_Read;
  HiByte := OW_Read;
  Return(Dummy);
end;


{-----------------------Nameplate Code-------------------------------------}

Procedure Nameplate_WriteCompany(Serial: t_OW_Serial; Var Company : t_Factory);
Var
  Authorization: t_Authorization;
Begin
  OW_SelectChip(Serial);
  OW_WritePage(0, @Company);
  OW_SelectChip(Serial);
  OW_ReadPage(0, NIL,Authorization);
  OW_SelectChip(Serial);
  DS2482_WriteByte(Byte(CopyPage));
  DS2482_WriteByte(Authorization.TA1);
  DS2482_WriteByte(Authorization.TA2);
  DS2482_WriteByte(Authorization.ES);
  MDelay(5);
  OW_SelectChip(Serial);
  OW_WritePage(32, @Company + 32);
  OW_SelectChip(Serial);
  OW_ReadPage(32, NIL,Authorization);
  OW_SelectChip(Serial);
  DS2482_WriteByte(Byte(CopyPage));
  DS2482_WriteByte(Authorization.TA1);
  DS2482_WriteByte(Authorization.TA2);
  DS2482_WriteByte(Authorization.ES);
  MDelay(5);
  OW_SelectChip(Serial);
  OW_WritePage(64, @Company + 64);
  OW_SelectChip(Serial);
  OW_ReadPage(64, NIL,Authorization);
  OW_SelectChip(Serial);
  DS2482_WriteByte(Byte(CopyPage));
  DS2482_WriteByte(Authorization.TA1);
  DS2482_WriteByte(Authorization.TA2);
  DS2482_WriteByte(Authorization.ES);
  MDelay(5);
  OW_SelectChip(Serial);
  OW_WritePage(96, @Company + 96);
  OW_SelectChip(Serial);
  OW_ReadPage(96, NIL,Authorization);
  OW_SelectChip(Serial);
  DS2482_WriteByte(Byte(CopyPage));
  DS2482_WriteByte(Authorization.TA1);
  DS2482_WriteByte(Authorization.TA2);
  DS2482_WriteByte(Authorization.ES);
  MDelay(5);
end;

Procedure Nameplate_ReadCompany(Serial: t_OW_Serial; Var Company : t_Factory);
Var
  Authorization: t_Authorization;
  Datapointer : Pointer;
  dummy : byte;
Begin
  OW_SelectChip(Serial);
  DS2482_WriteByte($f0);
  DS2482_WriteByte(00);
  DS2482_WriteByte(00);
  Datapointer := @Company;
  For dummy := 0 to 127 do
    datapointer^ := OW_Read;
    Inc(Datapointer);
  End_For;
end;

{--------------------------------------------------------------}
{ Main Program }
{$IDATA}

begin
  Start_Processes;
  LCDSetup_M(LCD_m1);
  LCDcursor_M(LCD_M1, false, false);
  Write(LCDout_M, 'E-LAB Computers');
  LCDXY_M(LCD_M1, 0, 1);
  BeepChirpH(4);

  With MyCompany do
    addresse.Company_Name := 'E-Lab Computers';
    addresse.Country := 'DE';
    addresse.ZipCode := '74906';
    addresse.Street := 'Grombacher Str. 27';
    addresse.city := 'Bad Rappenau';
    Telefon.Tel_Country_Code := '49';
    Telefon.Tel_Area_Code := '07268';
    Telefon.Tel_Number := '9124';
    Telefon.Tel_Ext := '0';
    Telefon.Fax_Country_Code := '49';
    Telefon.Fax_Area_Code := '07268';
    Telefon.Fax_Number := '9124';
    Telefon.Fax_Ext := '24';
  End_With;

  DS2482_SetI2CAdresse($1F);                                  // Set the I2C-Addr
  DS2482_Reset;                                               // Reset the I2C-Chip
  DS2482_SelectChannel(Channel_3);                            // Select Channel 3
  Check_OnLine := OW_Reset;                                   // False no Chip
  DS2482_SelectChannel(Channel_0);                            // Select Channel 1
  Check_OnLine := OW_Reset;                                   // True it has a Chip
  OW_ReadSingleSerial(Test);                                  // Read the DS2433 Serial
  Check:= OW_CalcCRC(Test);                                   // Calc the CRC
  Serial_OK := OW_MatchSerialCRC(Test);                       // Match the CRC
  
  if Serial_OK then
    BeepChirpL(4);
  else
    BeepSiren(0, 4);
  endif;

  Nameplate_WriteCompany(Test, MyCompany);
  Nameplate_ReadCompany(Test,  TestCompany);

  Write(LCDout_M, TestCompany.addresse.city);
  LCDXY_M(LCD_M1, 0, 2);
  Write(LCDout_M, TestCompany.addresse.street);

  WriteLn(SerOut, TestCompany.addresse.Company_Name);
  WriteLn(SerOut, TestCompany.addresse.Country);
  WriteLn(SerOut, TestCompany.addresse.ZipCode);
  WriteLn(SerOut, TestCompany.addresse.Street);
  WriteLn(SerOut, TestCompany.addresse.city);
  WriteLn(SerOut, TestCompany.Telefon.Tel_Country_Code);
  WriteLn(SerOut, TestCompany.Telefon.Tel_Area_Code);
  WriteLn(SerOut, TestCompany.Telefon.Tel_Number);
  WriteLn(SerOut, TestCompany.Telefon.Tel_Ext);
  WriteLn(SerOut, TestCompany.Telefon.Fax_Country_Code);
  WriteLn(SerOut, TestCompany.Telefon.Fax_Area_Code);
  WriteLn(SerOut, TestCompany.Telefon.Fax_Number);
  WriteLn(SerOut, TestCompany.Telefon.Fax_Ext);
  WriteLn(SerOut);

  DS2482_SelectChannel(Channel_1);                            // Selcet Channel 1
  DS1821_Setup;                                               // Set Free Mode
  DS1821_StartConv;                                           // Start the Operation
  loop
    Th := DS1821_GetTemperature;                              // Read the Temperature
    Write(SerOut, IntToStr(Th) +' ');
    LCDXY_M(LCD_M1, 0, 3);
    Write(LCDout_M, IntToStr(Th));
    Write(LCDout_M, ' '+IntToHex(Th));
    BeepClick;
    mDelay(500);
  endloop;
end I2C_1Wire.


